#include "../stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "wincompat.h"

#if defined(_WINDOWS) && !defined(WINCE)
#include <aclapi.h>
#endif

#ifdef _MSC_EXTENSIONS
#ifndef EAGAIN
#define EAGAIN          11
#endif
#define ENOMEM          12
#define ENOSYS          40
#endif


PSECURITY_ATTRIBUTES pSA = NULL;
#if defined(_WINDOWS) && !defined(WINCE)

/*
 * We have to set up a restrictive SECURITY_ATTRIBUTES for thread creation
 * so that Mono on Windows will not hang during shutdown.
 * Mono calls OpenThread(THREAD_ALL_ACCESS, TRUE, threadId); on any native
 * threads that it's seen via callbacks - we want this to fail, so we don't
 * allow the full THREAD_ALL_ACCESS access. fixes Unity3D hang bug.
 */

int useThreadSecurity = PTRUE;

SECURITY_ATTRIBUTES sa;
PSID pEveryoneSID = NULL;
PACL pACL = NULL;
PSECURITY_DESCRIPTOR pSD = NULL;

void destroyThreadSecurityAttributes()
{
	if (pEveryoneSID)
	{
        FreeSid(pEveryoneSID);
		pEveryoneSID = NULL;
	}
    if (pACL) 
	{
        LocalFree(pACL);
		pACL = NULL;
	}
	if (pSD) 
	{   
		LocalFree(pSD);
		pSD = NULL;
	}
	pSA = NULL;
}

void initializeThreadSecurityAttributes()
{
	DWORD dwRes;
    EXPLICIT_ACCESS ea[1];
    SID_IDENTIFIER_AUTHORITY SIDAuthWorld = SECURITY_WORLD_SID_AUTHORITY;

	if(pSA != NULL || !useThreadSecurity)
		return;

    // Create a well-known SID for the Everyone group.
    if(!AllocateAndInitializeSid(&SIDAuthWorld, 1,
                     SECURITY_WORLD_RID,
                     0, 0, 0, 0, 0, 0, 0,
                     &pEveryoneSID))
    {
        LOG(PHIDGET_LOG_ERROR, "AllocateAndInitializeSid Error %u\n", GetLastError());
        destroyThreadSecurityAttributes();
		return;
    }

    // Initialize an EXPLICIT_ACCESS structure for an ACE.
    // The ACE will limit access to everyone, so that OpenThread(THREAD_ALL_ACCESS) will fail from Mono
    ZeroMemory(&ea, sizeof(EXPLICIT_ACCESS));
    ea[0].grfAccessPermissions = STANDARD_RIGHTS_READ | THREAD_GET_CONTEXT | THREAD_QUERY_INFORMATION | STANDARD_RIGHTS_EXECUTE | SYNCHRONIZE;
    ea[0].grfAccessMode = SET_ACCESS;
    ea[0].grfInheritance= NO_INHERITANCE;
    ea[0].Trustee.TrusteeForm = TRUSTEE_IS_SID;
    ea[0].Trustee.TrusteeType = TRUSTEE_IS_WELL_KNOWN_GROUP;
    ea[0].Trustee.ptstrName  = (LPTSTR) pEveryoneSID;

    // Create a new ACL that contains the new ACEs.
    dwRes = SetEntriesInAcl(1, ea, NULL, &pACL);
    if (ERROR_SUCCESS != dwRes) 
    {
        printf("SetEntriesInAcl Error %u\n", GetLastError());
        destroyThreadSecurityAttributes();
		return;
    }

    // Initialize a security descriptor.  
    pSD = (PSECURITY_DESCRIPTOR) LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH); 
    if (NULL == pSD) 
    { 
        printf("LocalAlloc Error %u\n", GetLastError());
        destroyThreadSecurityAttributes();
		return;
    } 
 
    if (!InitializeSecurityDescriptor(pSD,
            SECURITY_DESCRIPTOR_REVISION)) 
    {  
        printf("InitializeSecurityDescriptor Error %u\n", GetLastError());
        destroyThreadSecurityAttributes();
		return;
    } 
 
    // Add the ACL to the security descriptor. 
    if (!SetSecurityDescriptorDacl(pSD, 
            TRUE,     // bDaclPresent flag   
            pACL, 
            FALSE))   // not a default DACL 
    {  
        printf("SetSecurityDescriptorDacl Error %u\n", GetLastError());
        destroyThreadSecurityAttributes();
		return;
    } 

    // Initialize a security attributes structure.
    sa.nLength = sizeof (SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = pSD;
    sa.bInheritHandle = FALSE;

	pSA = &sa;
}

#else
#define initializeThreadSecurityAttributes()
#endif

struct pthread {
	void *pt_handle;
	unsigned long pt_id;
};

struct pthread_cond {
	HANDLE ptc_handle;
};

struct pthread_mutex {
	CRITICAL_SECTION ptm_cs;
};

pthread_t pthread_self(void)
{
	return (pthread_t)GetCurrentThreadId();
}

int
pthread_create(pthread_t *pt, const pthread_attr_t *attr,
    void *(*start)(void *), void *arg)
{
	if (!(*pt = malloc(sizeof (**pt))))
		return errno = ENOMEM;
	initializeThreadSecurityAttributes();
	(*pt)->pt_handle = CreateThread(pSA, 0, (LPTHREAD_START_ROUTINE)start,
	    arg, 0, &(*pt)->pt_id);
	if (!(*pt)->pt_id)
		return errno = EAGAIN;

	return 0;
}

int pthread_up_priority(pthread_t pt)
{
	BOOL resp;
	DWORD err;
	resp = SetThreadPriority(pt->pt_handle, THREAD_PRIORITY_ABOVE_NORMAL);
	if(!resp)
		err = GetLastError();
	return 0;
}

int
pthread_join(pthread_t pt, void **status)
{
	DWORD ec;

	while (GetExitCodeThread(pt->pt_handle, &ec) && ec == STILL_ACTIVE)
		Sleep(100);
	if (status)
		*(DWORD *)status = ec;
	
	CloseHandle(pt->pt_handle);
	pt->pt_handle = NULL;

	return 0;
}

int
pthread_mutex_init(pthread_mutex_t *ptm, const pthread_mutexattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*ptm = malloc(sizeof (**ptm))))
		return errno = ENOMEM;
	InitializeCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_destroy(pthread_mutex_t *ptm)
{
	DeleteCriticalSection(&(*ptm)->ptm_cs);
	free(*ptm); *ptm = NULL;
	return 0;
}

int
pthread_mutex_lock(pthread_mutex_t *ptm)
{
	EnterCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_unlock(pthread_mutex_t *ptm)
{
	LeaveCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_cond_init(pthread_cond_t *cond, const pthread_condattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*cond = malloc(sizeof (**cond))))
		return errno = ENOMEM;
		
	(*cond)->ptc_handle = CreateEvent(NULL, FALSE, FALSE, NULL);
	return 0;
}

int
pthread_cond_destroy(pthread_cond_t *cond)
{
	CloseHandle((*cond)->ptc_handle);
	free(*cond); *cond = NULL;
	return 0;
}

int
pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
{
	int res;

	pthread_mutex_unlock(mutex);
	res = WaitForSingleObject((*cond)->ptc_handle, INFINITE);
	pthread_mutex_lock(mutex);
	if (res != WAIT_OBJECT_0)
		return errno = ENOSYS;
	return 0;
}

int
pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, int *timeout)
{
	int res;

	pthread_mutex_unlock(mutex);
	res = WaitForSingleObject((*cond)->ptc_handle, *timeout);
	pthread_mutex_lock(mutex);
	switch(res)
	{
		case WAIT_OBJECT_0:
			return 0;
		case WAIT_TIMEOUT:
			return ETIMEDOUT;
		case WAIT_FAILED:
		case WAIT_ABANDONED:
		default:
			return EINVAL;
	}
}

int
pthread_cond_signal(pthread_cond_t *ptc)
{
	ResetEvent((*ptc)->ptc_handle);
	SetEvent((*ptc)->ptc_handle);
	return 0;
}

int
usleep(unsigned long useconds)
{
	Sleep(useconds / 1000);
	return 0;
}

void
wsa_set_errdesc(char *errdesc, int len)
{
	int err;

	if (!errdesc)
		return;
	switch (err = WSAGetLastError()) {
	case 10038:
		snprintf(errdesc, len, "Socket operation on non-socket");
		return;
	case 10061:
		snprintf(errdesc, len, "Connection refused");
		return;
	default:
		snprintf(errdesc, len, "WSA error %d", err);
		return;
	}
}

/*socketpair for windows, remember to run psocket_init()*/
/*int socketpair(int domain, int type, int protocol, int sv[2]){

    if(protocol != 0){
	            errno = EINVAL;
	return -1;
	}
    if(type != SOCK_STREAM){
	        errno = EINVAL;
	return -1;
	}
    if(domain != AF_UNIX){
	          errno = EINVAL;
	return -1;
	}

    return _pipe(sv, 4096, _O_BINARY );
}*/

int socketpair(int domain, int type, int protocol, int sv[2]){
	int err, port;
	struct sockaddr_storage name;
	socklen_t namelen = sizeof(name);

	// Declare variables
	SOCKET ListenSocket, SendSocket, AcceptSocket;
	struct sockaddr_in saServer;

    if(protocol != 0){
		errno = 0;
		return -1;
	}
    if(type != SOCK_STREAM){
		errno = 0;
		return -1;
	}
    if(domain != AF_UNIX){
		errno = 0;
		return -1;
	}

	//We must use AF_INET, because Windows doesn't support AF_LOCAL sockets

	// Create a listening socket
	ListenSocket = socket(AF_INET, type, protocol);
	if(ListenSocket == INVALID_SOCKET)
	{
		errno = WSAGetLastError();
		return -1;
	}

	// Set up the sockaddr structure
	saServer.sin_family = AF_INET;
	saServer.sin_addr.s_addr = inet_addr( "127.0.0.1" );
	saServer.sin_port = htons(0); //choose a port

	// Bind the listening socket using the
	// information in the sockaddr structure
	bind( ListenSocket,(SOCKADDR*) &saServer, sizeof(saServer) );
	listen( ListenSocket, 1 );

	//Figure out the port
	if(getsockname(ListenSocket, (struct sockaddr *)&name, &namelen) != 0)
	{
		return -1;
	}
	port = (int)((struct sockaddr_in *)&name)->sin_port;

	SendSocket = socket(AF_INET, type, protocol);
	if(SendSocket == INVALID_SOCKET)
		return -1;

	err = connect( SendSocket, (SOCKADDR*) &name, sizeof(saServer) );
	if(err == SOCKET_ERROR)
	{
		errno = WSAGetLastError();
		return -1;
	}

	AcceptSocket = accept( ListenSocket, NULL, NULL );
	if(AcceptSocket == INVALID_SOCKET)
	{
		errno = WSAGetLastError();
		return -1;
	}
	closesocket(ListenSocket);
	sv[0] = SendSocket;
	sv[1] = AcceptSocket;

	return 0;
}


int
strcasecmp(const char *s1, const char *s2)
{
	return lstrcmpi((LPCTSTR)s1, (LPCTSTR)s2);
}

#if ((defined(_WINDOWS) && defined(__GNUC__)) || (defined(WINCE) && _WIN32_WCE < 0x600))
char *
strdup(const char *s)
{
	char *d = NULL;
	int l;

	if (!(d = malloc(l = (strlen(s)+1))))
		return NULL;
	memcpy(d, s, l);
	return d;
}
#endif
#if defined(WINCE)
//very basic, don't support bases other then 10, or endptr, or exponents
__int64 _strtoi64(
   const char *nptr,
   char **endptr,
   int base 
)
{
	__int64 number = 0;
	int ptr = 0;
	int numberpart = 0, neg = 0;

	while(nptr[ptr])
	{
		if(nptr[ptr] == '-')
		{
			if(!numberpart)
				neg = 1;
			else
				break;//fail
		}
		else if(nptr[ptr] == '+')
		{
			if(!numberpart)
				neg = 0;
			else
				break;//fail
		}
		else if(nptr[ptr] >= '0' && nptr[ptr] <= '9')
		{
			numberpart = 1;
			number *= 10;
			number += (nptr[ptr]-'0');
		}
		else
			break;

		ptr++;
	}

	if(neg)
		number = -number;
	
	return number;
}
#endif
